import 'dart:convert';
import 'dart:io';

import 'package:device_info_plus/device_info_plus.dart';
import 'package:flutter_gb_authentication_basic/flutter_gb_authentication_basic.dart';

class AuthDeviceInfoModel implements AuthDeviceInfo {
  @override
  String id;
  @override
  String name;

  AuthDeviceInfoModel({
    required this.id,
    required this.name,
  });

  static Future<AuthDeviceInfoModel> create() async {
    final deviceInfo = DeviceInfoPlugin();
    String id;
    String name;

    if (Platform.isAndroid) {
      final _info = (await deviceInfo.androidInfo);
      id = _info.id;
      if (id == "unknown") {
        id = generateUuid();
      }
      name = "${_info.manufacturer} ${_info.model} ${_info.product} - ${_info.version}";
    } else if (Platform.isIOS) {
      final _info = (await deviceInfo.iosInfo);
      id = _info.identifierForVendor ?? generateUuid();
      name = "${_info.model} ${_info.name} - ${_info.utsname.sysname} ${_info.utsname.release}";
    } else {
      id = generateUuid();
      name = "unknown";
    }

    return AuthDeviceInfoModel(
      id: id,
      name: name,
    );
  }

  @override
  String toJson() {
    final data = toMap();
    return jsonEncode(data);
  }

  @override
  Map<String, dynamic> toMap() {
    return {
      "id": id,
      "name": name,
    };
  }

  static AuthDeviceInfoModel fromJson(String deviceInfoData) {
    final data = jsonDecode(deviceInfoData);
    return AuthDeviceInfoModel(
      id: data["id"],
      name: data["name"],
    );
  }
}
